/* Characters Map/2
 *
 * Application window
 * Copyright (C) 2001 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: cmap_application.h,v 1.19 2001/10/10 14:07:27 glass Exp $
 */

/**@#-*/
#ifndef _CMAP_APPLICATION_H
#define _CMAP_APPLICATION_H

#include "pm_frame.h"
#include "pm_combobox.h"
#include "cmap_status.h"
#include "cmap_grid.h"
#include "cmap_glyph.h"
#include "cmap_specs.h"
/**@#+*/

/**
 * Characters Map/2 application window.
 * @author Dmitry A Steklenev
 */

class CMapApplication: public PMFrame {

  typedef PMFrame inherited;

  public:
    /** Constructs the application window. */
    CMapApplication( SHORT id );
    /** Destructs  the application window. */
   ~CMapApplication();

    /** Sets the input focus to the window. */
    virtual CMapApplication& set_focus();
    /** Displays a state. */
    CMapApplication& set_state( const char* state, ... );

  public:
    /** Selects specified font. */
    CMapApplication& select_font( int pos );
    /** Selects specified font size. */
    CMapApplication& select_size( int pos );

  protected:
    /** Dispatch system native events. */
    virtual MRESULT dispatch( ULONG msg, MPARAM mp1, MPARAM mp2 );

  private:
    CMapStatus lbl_status;
    PMWindow   lbl_font;
    PMComboBox cmb_font;
    PMWindow   lbl_size;
    PMComboBox cmb_size;
    CMapGrid   fnt_grid;
    PMWindow   lbl_codepage;
    PMComboBox cmb_codepage;
    CMapGlyph  fnt_glyph;
    PMWindow   btn_clear;
    PMWindow   btn_specs;
    PMWindow   btn_about;
    CMapSpecs* win_specs;

    HWND  mnu_handle;
    ULONG clb_flags;
};

/**@#-*/
#endif
