/* Characters Map/2
 *
 * Clipboard management
 * Copyright (C) 2001 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: cmap_clipboard.cpp,v 1.2 2001/10/10 14:07:27 glass Exp $
 */

#define  INCL_WIN
#define  INCL_ERRORS
#include <os2.h>
#include <stdlib.h>
#include <stdio.h>

#include "pm_error.h"
#include "pm_gui.h"
#include "cmap_clipboard.h"

#define  CF_RTF  "Rich Text Format"

/*--------------------------------------------------
 * Clear clipboard
 *--------------------------------------------------*/
void CMapClipboard::clear()
{
  if( !WinOpenClipbrd ( PMGUI::hab()))
    PM_THROW_GUIERROR();

  if( !WinEmptyClipbrd( PMGUI::hab()))
    PM_THROW_GUIERROR();

  if( !WinCloseClipbrd( PMGUI::hab()))
    PM_THROW_GUIERROR();
}

/*--------------------------------------------------
 * If the clipboard is empty, true is returned
 *--------------------------------------------------*/
BOOL CMapClipboard::is_empty()
{
  BOOL is_empty = TRUE;

  if( !WinOpenClipbrd ( PMGUI::hab()))
    PM_THROW_GUIERROR();

  if( WinQueryClipbrdData( PMGUI::hab(), CF_TEXT ))
    is_empty = FALSE;
  else
  {
    ATOM cf_rtf = WinFindAtom( WinQuerySystemAtomTable(), CF_RTF );
    is_empty    = !cf_rtf || !WinQueryClipbrdData( PMGUI::hab(), cf_rtf );
  }

  if( !WinCloseClipbrd( PMGUI::hab()))
    PM_THROW_GUIERROR();

  return is_empty;
}

/*--------------------------------------------------
 * Inserts the string as the last string
 *--------------------------------------------------*/
void CMapClipboard::push_back( const char* string,
                               const char* font  , ULONG cp, ULONG flags )
{
  if( !WinOpenClipbrd ( PMGUI::hab()))
    PM_THROW_GUIERROR();

  // Prepare current attributes

  char* clipboard_text = (char*)WinQueryClipbrdData( PMGUI::hab(), CF_TEXT );
  LONG  clipboard_len  = clipboard_text ? strlen(clipboard_text) : 0;

  // Prepare new ascii and RTF texts

  char* rtf_template = "{\\rtf1\\ansi{\\fonttbl{\\f0\\fnil \\cpg%u %s;}} \\f0 ";
  char* asc_data     = 0;
  char* rtf_data     = 0;

  ATOM cf_rtf = WinFindAtom( WinQuerySystemAtomTable(), CF_RTF );
  if( !cf_rtf ) cf_rtf = WinAddAtom( WinQuerySystemAtomTable(), CF_RTF );
  if( !cf_rtf ) PM_THROW_GUIERROR();

  APIRET rc;

  rc = DosAllocSharedMem((void**)&asc_data, NULL,
                         clipboard_len+strlen(string)+1,
                         PAG_WRITE | OBJ_GIVEABLE | PAG_COMMIT );
  if( rc != NO_ERROR )
    PM_THROW_OS2ERROR(rc);

  rc = DosAllocSharedMem((void**)&rtf_data, NULL,
                         clipboard_len*4+strlen(font)+strlen(rtf_template)+strlen(string)+1,
                         PAG_WRITE | OBJ_GIVEABLE | PAG_COMMIT );
  if( rc != NO_ERROR )
    PM_THROW_OS2ERROR(rc);

  if( clipboard_text )
    strcpy( asc_data, clipboard_text );

  strcat ( asc_data, string );
  sprintf( rtf_data, rtf_template, cp, font );

  char* trg = rtf_data + strlen(rtf_data);
  char* src = asc_data;

  while( *src ) {
    sprintf( trg, "\\'%02x", *src++ );
    trg += 4;
  }

  *trg++ = '}';
  *trg   = 0;

  // Clear clipboard and set new data

  if( !WinEmptyClipbrd( PMGUI::hab()))
    PM_THROW_GUIERROR();

  if( flags & rtf   )
    if( !WinSetClipbrdData( PMGUI::hab(), (ULONG)rtf_data, cf_rtf , CFI_POINTER ))
      PM_THROW_GUIERROR();

  if( flags & ascii )
    if( !WinSetClipbrdData( PMGUI::hab(), (ULONG)asc_data, CF_TEXT, CFI_POINTER ))
      PM_THROW_GUIERROR();

  if( !WinCloseClipbrd( PMGUI::hab()))
    PM_THROW_GUIERROR();
}

