/* Characters Map/2
 *
 * Fonts management class
 * Copyright (C) 2001 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: cmap_fonts.h,v 1.7 2002/09/12 08:36:58 glass Exp $
 */

/**@#-*/
#ifndef _CMAP_FONTS_H
#define _CMAP_FONTS_H
/**@#+*/

/**
 * Characters Map/2 fonts management class.
 *
 * Provide support for managing system fonts.
 * @author Dmitry A Steklenev
 */

class CMapFonts {

  public:
    /** Returns available font metrics. */
    static FONTMETRICS* metrics();
    /** Returns number of font metrics available. */
    static ULONG metrics_count ();

    /**
     * Refreshes font metrics list.
     *
     * Updates the list of font metrics mirroring the last changes
     * in the system. After that the pointer obtained earlier from
     * metrics() can be invalid.
     */

    static void refresh();

  private:
    static FONTMETRICS* fonts_metrics;
    static ULONG        count_metrics;
};

#if !defined(__ccdoc__)
/*--------------------------------------------------
 * Returns number of font metrics available
 *--------------------------------------------------*/
inline ULONG CMapFonts::metrics_count()
{
  return count_metrics;
}
#endif
/**@#-*/
#endif
