/* Characters Map/2
 *
 * The glyph window class
 * Copyright (C) 2001 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: cmap_glyph.h,v 1.6 2002/09/12 08:36:58 glass Exp $
 */

/**@#-*/
#ifndef _CMAP_GLYPH_H
#define _CMAP_GLYPH_H

#include "pm_window.h"
/**@#+*/

/**
 * Characters Map/2 glyph window class.
 * @author Dmitry A Steklenev
 */

class CMapGlyph: public PMWindow {

  typedef PMWindow inherited;

  public:
    /** Constructs the glyph window. */
    CMapGlyph( SHORT id, HWND parent, HWND owner, long x, long y, long cx, long cy );
    /** Destructs the glyph window. */
   ~CMapGlyph();

    /** Selects the displayed font. */
    CMapGlyph& select_font( const FATTRS& fonts );
    /** Returns the selected character code. */
    char selected() const;
    /** Selects the specified character code. */
    CMapGlyph& select( char code );

  protected:
    /** Dispatch system native events. */
    virtual MRESULT dispatch( ULONG msg, MPARAM mp1, MPARAM mp2 );

  private:
    CMapGlyph& draw_it();

    static const long glyph_font_id;

    HDC     glyph_dc;
    HPS     glyph_ps;
    HBITMAP glyph_bitmap;
    char    glyph_selected;
};

#if !defined(__ccdoc__)
/*--------------------------------------------------
 * Returns the selected character code
 *--------------------------------------------------*/
inline char CMapGlyph::selected() const
{
  return glyph_selected;
}
#endif
/**@#-*/
#endif
