/* Characters Map/2
 *
 * The grid window class
 * Copyright (C) 2001 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: cmap_grid.h,v 1.11 2002/09/12 08:36:58 glass Exp $
 */

/**@#-*/
#ifndef _CMAP_GRID_H
#define _CMAP_GRID_H

#include "pm_window.h"
#include "pm_thread.h"
#include "pm_exclusion.h"
/**@#+*/

/**
 * Characters Map/2 grid window class.
 * @author Dmitry A Steklenev
 */

class CMapGrid: public PMWindow, protected PMThread {

  typedef PMWindow inherited;

  public:
    /** Constructs the grid window. */
    CMapGrid( SHORT id, HWND parent, HWND owner, long x, long y );
    /** Destructs the grid window. */
   ~CMapGrid();

    /** Selects the displayed font. */
    CMapGrid& select_font( const FATTRS& fonts );
    /** Returns the selected character code. */
    char selected() const;
    /** Selects the specified character code. */
    CMapGrid& select( char code );

  protected:
    /** Dispatch system native events. */
    virtual MRESULT dispatch( ULONG msg, MPARAM mp1, MPARAM mp2 );
    /** Dispatch keyboard events. */
    virtual MRESULT dispatch_keyboard( ULONG msg, MPARAM mp1, MPARAM mp2 );

  private:
    void  run  ();
    void  clear();

    /** Returns calculated cell's rectangle. */
    SHAPE cell_shape( char code ) const;
    /** Returns calculated cell's rectangle. */
    SHAPE cell_shape(           ) const;

    static const long grid_cell_size;
    static const long grid_dimension;
    static const long grid_font_id;

    HDC         grid_dc;
    HPS         grid_ps;
    HBITMAP     grid_bitmap;
    HQUEUE      grid_queue;
    char        grid_selected;
    PMExclusion grid_sentry;
};

#if !defined(__ccdoc__)
/*--------------------------------------------------
 * Returns the selected character code
 *--------------------------------------------------*/
inline char CMapGrid::selected() const
{
  return grid_selected;
}

/*--------------------------------------------------
 * Returns calculated cell's rectangle
 *--------------------------------------------------*/
inline SHAPE CMapGrid::cell_shape() const
{
  return cell_shape( grid_selected );
}
#endif

/**@#-*/
#endif
