/* Combination Box window class
 * Copyright (C) 2000 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: pm_combobox.cpp,v 1.10 2001/06/08 17:25:45 glass Exp $
 */

#define  INCL_WIN
#include <os2.h>

#include "pm_combobox.h"

/*--------------------------------------------------
 * Constructs the combination box window
 *--------------------------------------------------*/
PMComboBox::PMComboBox( SHORT id, HWND parent, HWND  owner,
                        long  x, long y, long cx, long cy )

: inherited(  WC_COMBOBOX, id, "", parent, owner, x, y, cx, cy,
              WS_VISIBLE | WS_TABSTOP | WS_GROUP | CBS_DROPDOWNLIST )
{}

/*--------------------------------------------------
 * Inserts the string
 *--------------------------------------------------*/
int PMComboBox::insert( const char* string, ULONG hitem, int before )
{
  int pos = (int)WinSendMsg( handle(), LM_INSERTITEM,
                             MPFROMSHORT(before),
                             MPFROMP(string));

  if( pos == LIT_ERROR || pos == LIT_MEMERROR )
    PM_THROW_GUIERROR();

  if( hitem )
    set_item_handle( pos, hitem );

  return pos;
}

/*--------------------------------------------------
 * Searches the specified string
 *--------------------------------------------------*/
int PMComboBox::search( const char* string ) const
{
  int pos = (int)WinSendMsg( handle(),
                             LM_SEARCHSTRING,
                             MPFROM2SHORT( 0, LIT_FIRST ),
                             MPFROMP(string));
  if( pos == LIT_ERROR )
    PM_THROW_GUIERROR();

  return pos;
}

/*--------------------------------------------------
 * Sets the selection state of an item
 *--------------------------------------------------*/
PMComboBox& PMComboBox::select( int pos, BOOL state )
{
  if( pos != LIT_NONE )
    if( !WinSendMsg( handle(), LM_SELECTITEM, MPFROMSHORT(pos), MPFROMLONG(state)))
      PM_THROW_GUIERROR();

  return *this;
}

/*--------------------------------------------------
 * Returns the selected item
 *--------------------------------------------------*/
int PMComboBox::selected() const
{
  return (int)WinSendMsg( handle(), LM_QUERYSELECTION,
                          MPFROMLONG(LIT_FIRST), 0 );
}

/*--------------------------------------------------
 * Returns the item string
 *--------------------------------------------------*/
char* PMComboBox::string( int pos, char* buffer, int buf_len ) const
{
  WinSendMsg( handle(), LM_QUERYITEMTEXT,
              MPFROM2SHORT( pos, buf_len), MPFROMP(buffer));

  return buffer;
}

/*--------------------------------------------------
 * Returns the number of strings
 *--------------------------------------------------*/
int PMComboBox::count() const
{
  return (int)WinSendMsg( handle(), LM_QUERYITEMCOUNT, 0, 0 );
}

/*--------------------------------------------------
 * Removes all items from the combination box
 *--------------------------------------------------*/
PMComboBox& PMComboBox::clear()
{
  if( !WinSendMsg( handle(), LM_DELETEALL, 0, 0 ))
    PM_THROW_GUIERROR();

  return *this;
}

/*--------------------------------------------------
 * Sets the handle of the specified item
 *--------------------------------------------------*/
PMComboBox& PMComboBox::set_item_handle( int pos, ULONG hitem )
{
  if( !WinSendMsg( handle(), LM_SETITEMHANDLE, MPFROMSHORT(pos), MPFROMLONG(hitem)))
    PM_THROW_GUIERROR();

  return *this;
}

/*--------------------------------------------------
 * Returns the handle of the specified item
 *--------------------------------------------------*/
ULONG PMComboBox::item_handle( int pos ) const
{
  return (ULONG)WinSendMsg( handle(), LM_QUERYITEMHANDLE, MPFROMSHORT(pos), 0 );
}

