/* Combination box window class
 * Copyright (C) 2001 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: pm_combobox.h,v 1.11 2001/10/10 13:39:39 glass Exp $
 */

/**@#-*/
#ifndef _PM_COMBOBOX_H
#define _PM_COMBOBOX_H

#include "pm_window.h"
/**@#+*/

/**
 * Combination box window class.
 *
 * The PMComboBox class creates and manages combination box control
 * windows. The PMBaseComboBox class combines an entry field and a
 * list box to form one control containing the features of both.
 *
 * @author Dmitry A Steklenev
 */

class PMComboBox : public PMWindow
{
  typedef PMWindow inherited;

  public:
    /** Constructs the combination box window. */
    PMComboBox( SHORT id, HWND parent, HWND owner, long x, long y,long cx, long cy );

  public:
    /** Inserts the string. */
    int insert( const char* string, ULONG handle = 0, int pos = LIT_SORTASCENDING );

    /** Searches the specified string. */
    int search( const char* string ) const;
    /** Sets the selection state of an item. */
    PMComboBox& select( int pos, BOOL state = TRUE );

    /** Returns the item string. */
    char* string( int pos, char* buffer, int buf_len ) const;
    /** Returns the selected item or LIT_NONE. */
    int selected() const;
    /** Returns the number of strings stored in the combination box. */
    int count() const;
    /** Removes all items from the combination box. */
    PMComboBox& clear();

    /** Sets the handle of the specified combination box item. */
    PMComboBox& set_item_handle( int pos, ULONG handle );
    /** Returns the handle of the specified combination box item. */
    ULONG item_handle( int pos ) const;
};

/**@#-*/
#endif