/* Base error class
 * Copyright (C) 2000 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: pm_error.cpp,v 1.6 2001/06/08 17:25:45 glass Exp $
 */

#define  INCL_WIN
#define  INCL_DOS
#define  INCL_ERRORS
#include <os2.h>

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "pm_gui.h"
#include "pm_error.h"

extern const char* application_name;

/*--------------------------------------------------
 * Constructs error from current GUI error
 *--------------------------------------------------*/
PMError::PMError( const char* file, const char* func, int line )

: err_file( strdup( file )),
  err_func( strdup( func )),
  err_line( line ),
  err_code( 0    ),
  err_info( NULL )
{
  PERRINFO os2_info = WinGetErrorInfo( PMGUI::hab());

  if( os2_info )
  {
    err_code = os2_info->idError;
    err_info = (char*)os2_info +
                     ((char*)os2_info + os2_info->offaoffszMsg)[0];
  }

  err_info = strdup( err_info );

  if( os2_info )
    WinFreeErrorInfo(os2_info);
}

/*--------------------------------------------------
 * Constructs error from specified error id and information
 *--------------------------------------------------*/
PMError::PMError( int code, const char* info,
                            const char* file, const char* func, int line )
: err_file( strdup( file )),
  err_func( strdup( func )),
  err_line( line ),
  err_code( code )
{
  if( !info )
  {
    char   szErrInfo[1024];
    ULONG  ulMessageLength;
    APIRET rc;

    rc = DosGetMessage( 0, 0, (PCHAR)szErrInfo, sizeof(szErrInfo),
                        code, (PSZ)"OSO001.MSG", &ulMessageLength );

    info = rc ? "No error text is available" : szErrInfo;
  }

  err_info = strdup( info );
}

/*--------------------------------------------------
 * Copy constructor
 *--------------------------------------------------*/
PMError::PMError( const PMError& error )

: err_file( strdup( error.err_file )),
  err_func( strdup( error.err_func )),
  err_info( strdup( error.err_info )),
  err_line( error.err_line ),
  err_code( error.err_code )

{}

/*--------------------------------------------------
 * Assignment operator
 *--------------------------------------------------*/
PMError& PMError::operator=( const PMError& error )
{
  err_line = error.err_line;
  err_code = error.err_code;
  err_file = strdup( error.err_file );
  err_func = strdup( error.err_func );
  err_info = strdup( error.err_info );

  return *this;
}

/*--------------------------------------------------
 * Destructor
 *--------------------------------------------------*/
PMError::~PMError()
{
  free( err_info );
  free( err_func );
  free( err_file );
}

/*--------------------------------------------------
 * Display error information
 *--------------------------------------------------*/
PMError& PMError::display( FILE* out )
{
  const char* msg_fmt = "Unexpected error at:\n"
                        "  file: %s\n"
                        "  function: %s\n"
                        "  line: %u\n\n"
                        "Error Id is: %u (%x)\n"
                        "Exception text is:\n\n"
                        "%s\n";

  char* msg_buf = new char[ strlen(msg_fmt) +
                            strlen(file() ) +
                            strlen(func() ) +
                            strlen(info() ) + 64 ];

  char* msg_ttl = new char[71];

  sprintf( msg_buf, msg_fmt, file(), func(), line(), code(), code(), info());
  sprintf( msg_ttl, "%-070s", application_name );

  if( out )
  {
    fprintf( out, msg_buf );
    fflush ( out );
  }
  else
    WinMessageBox( HWND_DESKTOP, HWND_DESKTOP, msg_buf, msg_ttl, 100,
                   MB_OK | MB_ERROR | MB_APPLMODAL | MB_MOVEABLE );

  delete[] msg_buf;
  delete[] msg_ttl;
  return *this;
}

