/* Serialize access to resources
 * Copyright (C) 2001 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: pm_exclusion.cpp,v 1.1 2001/06/08 17:25:45 glass Exp $
 */

#define  INCL_DOSSEMAPHORES
#include <os2.h>

#include "pm_exclusion.h"

/*--------------------------------------------------
 * Constructor
 *-------------------------------------------------*/
PMExclusion::PMExclusion()
{
  DosCreateMutexSem( NULL, &exc_handle, 0, 0 );
}

/*--------------------------------------------------
 * Destructor
 *-------------------------------------------------*/
PMExclusion::~PMExclusion()
{
  DosCloseMutexSem( exc_handle );
}

/*--------------------------------------------------
 * Request of a resource
 *-------------------------------------------------*/
BOOL PMExclusion::request()
{
  return !DosRequestMutexSem( exc_handle, SEM_INDEFINITE_WAIT );
}

/*--------------------------------------------------
 * Request of a resource with timeout
 *-------------------------------------------------*/
BOOL PMExclusion::request( unsigned long mseq )
{
  return !DosRequestMutexSem( exc_handle, mseq );
}

/*--------------------------------------------------
 * Release of a resource
 *-------------------------------------------------*/
BOOL PMExclusion::release()
{
  return !DosReleaseMutexSem( exc_handle );
}

