/* Frame window class
 * Copyright (C) 2001 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: pm_frame.cpp,v 1.1 2001/06/08 17:25:45 glass Exp $
 */

#define  INCL_WIN
#include <os2.h>

#include "pm_frame.h"

extern const char* application_name;
static FRAMECDATA  fcdata = { sizeof( FRAMECDATA ), 0, 0, 0 };

/*--------------------------------------------------
 * Constructs the frame window
 *--------------------------------------------------*/
PMFrame::PMFrame( SHORT       id,
                  const char* caption,
                  HWND        owner,
                  long        x,
                  long        y,
                  long        cx,
                  long        cy,
                  ULONG       style )

: inherited ( WC_FRAME, id, caption, HWND_DESKTOP, owner,
              x, y, cx, cy, WS_SAVEBITS, ( fcdata.flCreateFlags = style, &fcdata )),
  frm_client( 0 )
{
  // Change window background
  WinSetWindowUShort( handle(), QWS_FLAGS,
    WinQueryWindowUShort( handle(), QWS_FLAGS ) | FF_DIALOGBOX );

  // Assign icon
  HPOINTER hIcon = WinLoadPointer( HWND_DESKTOP, NULLHANDLE, id );

  if( hIcon == NULLHANDLE  ||
     !WinSendMsg( handle(), WM_SETICON, MPFROMLONG(hIcon), 0 ))
  {
    PM_THROW_GUIERROR();
  }

  if( !WinSetWindowPos( handle(), HWND_TOP, 0, 0, 0, 0, SWP_ZORDER ))
    PM_THROW_GUIERROR();

  // Needed for correct repainting of a frame controls
  WinSendMsg( handle(), WM_UPDATEFRAME,
                        MPFROMLONG(fcdata.flCreateFlags), 0 );
}

/*--------------------------------------------------
 * Calculate client rectangle
 *--------------------------------------------------*/
SHAPE PMFrame::client_shape() const
{
   SHAPE shape = this->shape();
   RECTL rect  = { 0, 0, shape.cx, shape.cy };

   WinCalcFrameRect( handle(), &rect, TRUE );

   shape.x  = rect.xLeft;
   shape.y  = rect.yBottom;
   shape.cx = rect.xRight - rect.xLeft;
   shape.cy = rect.yTop   - rect.yBottom;

   return shape;
}

/*--------------------------------------------------
 * Save the current position and size of the window
 *--------------------------------------------------*/
PMFrame& PMFrame::save( const char* key )
{
  WinStoreWindowPos( application_name, key, handle());
  return *this;
}

/*--------------------------------------------------
 * Restore the current position and size of the window
 *--------------------------------------------------*/
BOOL PMFrame::load( const char* key )
{
  return WinRestoreWindowPos( application_name, key, handle());
}

/*--------------------------------------------------
 * Sets the client window
 *--------------------------------------------------*/
PMFrame& PMFrame::set_client( HWND client )
{
  frm_client = client;

  if( frm_client )
    PMWindow( frm_client ).reshape( client_shape());

  return *this;
}

/*--------------------------------------------------
 * Dispatch system native events
 *--------------------------------------------------*/
MRESULT PMFrame::dispatch( ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  switch( msg )
  {
    case WM_ERASEBACKGROUND:
      if( frm_client )
        return 0;

      break;

    case WM_FORMATFRAME:
      if( frm_client )
      {
        MRESULT rc = dispatch_everything( msg, mp1, mp2 );
        PMWindow( frm_client ).reshape( client_shape());
        return  rc;
      }
      break;
  }
  return inherited::dispatch( msg, mp1, mp2 );
}

