/* Frame window class
 * Copyright (C) 2001 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: pm_frame.h,v 1.4 2001/10/31 10:01:04 glass Exp $
 */

/**@#-*/
#ifndef _PM_FRAME_H
#define _PM_FRAME_H

#include "pm_window.h"
/**@#+*/

/**
 * Frame window class.
 *
 * A composite window class that can contain child windows
 * of many of the other window classes.
 *
 * @author Dmitry A Steklenev
 */

class PMFrame: public PMWindow {

  typedef PMWindow inherited;

  public:
    /** Constructs the frame window. */
    PMFrame( SHORT       id,
             const char* caption,
             HWND        owner,
             long        x,
             long        y,
             long        cx,
             long        cy,
             ULONG       style  );
  public:

    /** Calculates client shape. */
    virtual SHAPE client_shape() const;
    /** Sets the client window. */
    virtual PMFrame& set_client( HWND client );

    /**
     * Save the current position and size of the window.
     *
     * This method will also save the presentation parameters
     * of the window and all children.
     */

    virtual PMFrame& save( const char* key );

    /**
     * Restore the position and size of the window.
     *
     * This method will also restore the presentation parameters
     * of the window and all children.
     */

    virtual BOOL load( const char* key );

  protected:
    /** Dispatch system native events. */
    virtual MRESULT dispatch( ULONG msg, MPARAM mp1, MPARAM mp2 );

  private:
    HWND frm_client;
};

/**@#-*/
#endif