/* GUI initialization and information Class
 * Copyright (C) 2000 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: pm_gui.cpp,v 1.7 2001/09/24 09:52:56 glass Exp $
 */

#define  INCL_WIN
#define  INCL_DOS
#define  INCL_ERRORS
#include <os2.h>
#include <stdlib.h>

#include "pm_gui.h"
#include "pm_error.h"

HAB PMGUI::gui_hab[PM_MAX_THREADS];
HMQ PMGUI::gui_hmq[PM_MAX_THREADS];

/*--------------------------------------------------
 * Initializes GUI
 *--------------------------------------------------*/
PMGUI::PMGUI()
{
  TID cur_tid = tid();

  if( cur_tid > PM_MAX_THREADS )
    PM_THROW_ERROR( 0, "Too many threads" );

  gui_hab[cur_tid] = NULLHANDLE;
  gui_hmq[cur_tid] = NULLHANDLE;

  if( !(gui_hab[cur_tid] = WinInitialize(0)))
    PM_THROW_ERROR( 0, "Unexpected error in WinInitialize()" );

  if( !(gui_hmq[cur_tid] = WinCreateMsgQueue( gui_hab[cur_tid], 0 )))
    PM_THROW_GUIERROR();

  gui_wrapped = FALSE;
}

/*--------------------------------------------------
 * Wraps initialized GUI
 *--------------------------------------------------*/
PMGUI::PMGUI( HAB hab )
{
  TID cur_tid = tid();

  if( cur_tid > PM_MAX_THREADS )
    PM_THROW_ERROR( 0, "Too many threads" );

  gui_hab[cur_tid] = hab;
  gui_hmq[cur_tid] = HMQ_CURRENT;
  gui_wrapped      = TRUE;
}

/*--------------------------------------------------
 * Terminates GUI
 *--------------------------------------------------*/
PMGUI::~PMGUI()
{
  TID cur_tid = tid();

  if( !gui_wrapped )
  {
    if( gui_hmq[cur_tid] ) WinDestroyMsgQueue( gui_hmq[cur_tid] );
    if( gui_hab[cur_tid] ) WinTerminate      ( gui_hab[cur_tid] );

    gui_hab[cur_tid] = NULLHANDLE;
    gui_hmq[cur_tid] = NULLHANDLE;
  }
}

/*--------------------------------------------------
 * Returns the current thread identifier
 *--------------------------------------------------*/
TID PMGUI::tid()
{
  PTIB ptib;
  DosGetInfoBlocks( &ptib, NULL );
  return ptib->tib_ptib2->tib2_ultid;
}

/*--------------------------------------------------
 * Returns the current process identifier
 *--------------------------------------------------*/
PID PMGUI::pid()
{
  PPIB ppib;
  DosGetInfoBlocks( NULL, &ppib );
  return ppib->pib_ulpid;
}

/*--------------------------------------------------
 * Returns anchor block handle for current thread
 *--------------------------------------------------*/
HAB PMGUI::hab()
{
  return gui_hab[tid()];
}

/*--------------------------------------------------
 * Returns message queue handle for current thread
 *--------------------------------------------------*/
HMQ PMGUI::hmq()
{
  return gui_hmq[tid()];
}

/*--------------------------------------------------
 * Dispatch messages
 *--------------------------------------------------*/
void PMGUI::run()
{
  QMSG qms;
  HAB  cur_hab = hab();

  while( WinGetMsg( cur_hab, &qms, 0, 0, 0 ))
    WinDispatchMsg( cur_hab, &qms );
}
