/* GUI initialization and information class
 * Copyright (C) 2000 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: pm_gui.h,v 1.10 2001/10/10 13:39:39 glass Exp $
 */

/**@#-*/
#ifndef _PM_GUI_H
#define _PM_GUI_H

#ifndef PM_MAX_THREADS
#define PM_MAX_THREADS 20
#endif

#ifndef RGB_DARKGRAY
#define RGB_DARKGRAY   0x00808080L
#endif
#ifndef RGB_PALEGRAY
#define RGB_PALEGRAY   0x00CCCCCCL
#endif

#include "pm_noncopyable.h"
/**@#+*/

/**
 * GUI initialization and information class.
 *
 * Initialize GUI in current thread. Automatically called
 * from <i>PMThread</i>.
 *
 * @author Dmitry A Steklenev
 */

class PMGUI : public PMNonCopyable
{
  public:
    /** Initializes the GUI facilities for use by an application. */
    PMGUI();
    /** Constructs the object from a existing GUI environment. */
    PMGUI( HAB hab );
    /** Releases all of associated GUI resources */
   ~PMGUI();

    /** Returns the current thread identifier. */
    static TID  tid();
    /** Returns the current process identifier. */
    static PID  pid();
    /** Returns anchor block handle for current thread. */
    static HAB  hab();
    /** Returns message queue handle for current thread. */
    static HMQ  hmq();
    /** Dispatch messages. */
    static void run();

  private:
    BOOL   gui_wrapped;
    static HAB gui_hab[PM_MAX_THREADS];
    static HMQ gui_hmq[PM_MAX_THREADS];
};

/**@#-*/
#endif