/* Message catalog management class
 * Copyright (C) 2000 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: pm_messages.cpp,v 1.4 2001/06/08 17:25:46 glass Exp $
 */

#define  INCL_WIN
#include <os2.h>
#include <stdio.h>
#include <stdlib.h>

#include "pm_messages.h"
#include "pm_error.h"

nl_catd PMMsg::msg_catd;
int     PMMsg::msg_country;
PMMsg   PMMsg::msg_sentry;

/*--------------------------------------------------
 * Destructs PMMsg object
 *--------------------------------------------------*/
PMMsg::~PMMsg()
{
  if( msg_catd )
    catclose( msg_catd );
}

/*--------------------------------------------------
 * Open message catalog
 *--------------------------------------------------*/
void PMMsg::open_catalog( const char* catalog_name )
{
  msg_catd    = catopen( catalog_name, 0 );
  msg_country = 1;

  if( msg_catd == CATD_ERR )
  {
    char err_info[1024];
    sprintf( err_info, "Message catalog '%s' not found!", catalog_name );
    PM_THROW_ERROR( 0, err_info );
  }
  else
    select_country();
}

/*--------------------------------------------------
 * Select a country
 *--------------------------------------------------*/
void PMMsg::select_country( int country )
{
  if( !msg_catd )
    return;

  if( !country  )
  {
    char country_code[16] = "";
    ULONG len = sizeof(country_code) - 1;

    PrfQueryProfileData( HINI_USERPROFILE,
                         "PM_National", "iCountry", &country_code, &len );

    country_code[len] = '\0';
    msg_country = atoi( country_code );
  }
  else
    msg_country = country;

  if( catgets( msg_catd, msg_country, 1, NULL ) == NULL )
    msg_country = 1;
}

/*--------------------------------------------------
 * Get message
 *--------------------------------------------------*/
char* PMMsg::get( int msg_no )
{
  return catgets( msg_catd, msg_country, msg_no, "Message not found" );
}

