/* User profile management class
 * Copyright (C) 2000 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: pm_profile.cpp,v 1.2 2001/10/31 10:01:04 glass Exp $
 */

#define  INCL_WIN
#include <os2.h>
#include <string.h>

#include "pm_profile.h"

extern const char* application_name;

/*--------------------------------------------------
 * Puts data to user profile
 *--------------------------------------------------*/
void PMProfile::put_data( const char* key, const void* exp, unsigned long len )
{
  PrfWriteProfileData( HINI_USERPROFILE,
                       application_name,
                       key, (PVOID)exp, len );
}

/*--------------------------------------------------
 * Retrieves data from user profile
 *--------------------------------------------------*/
unsigned long PMProfile::get_data( const char* key, void* exp, unsigned long len )
{
  unsigned long get_len = len;

  PrfQueryProfileData( HINI_USERPROFILE,
                       application_name,
                       key, exp, &get_len );
  return get_len;
}

/*--------------------------------------------------
 * Puts long value to user profile
 *--------------------------------------------------*/
void PMProfile::put_long( const char *key, long exp )
{
  PrfWriteProfileData( HINI_USERPROFILE,
                       application_name,
                       key, &exp, sizeof(exp));
}

/*--------------------------------------------------
 * Retrieves long value from user profile
 *--------------------------------------------------*/
long PMProfile::get_long( const char *key, long def )
{
  long exp = def;
  unsigned long len = sizeof(exp);

  return PrfQueryProfileData( HINI_USERPROFILE,
                              application_name,
                              key, &exp, &len ) ? exp : def;
}

/*--------------------------------------------------
 * Puts unsigned long value to user profile
 *--------------------------------------------------*/
void PMProfile::put_ulong( const char *key, unsigned long exp )
{
  PrfWriteProfileData( HINI_USERPROFILE,
                       application_name,
                       key, &exp, sizeof(exp));
}

/*--------------------------------------------------
 * Retrieves unsigned long value from user profile
 *--------------------------------------------------*/
unsigned long PMProfile::get_ulong( const char *key, unsigned long def )
{
  unsigned long exp = def;
  unsigned long len = sizeof(exp);

  return PrfQueryProfileData( HINI_USERPROFILE,
                              application_name,
                              key, &exp, &len ) ? exp : def;
}

/*--------------------------------------------------
 * Puts short value to user profile
 *--------------------------------------------------*/
void PMProfile::put_short( const char *key, short exp )
{
  PrfWriteProfileData( HINI_USERPROFILE,
                       application_name,
                       key, &exp, sizeof(exp));
}

/*--------------------------------------------------
 * Retrieves short value from user profile
 *--------------------------------------------------*/
short PMProfile::get_short( const char *key, short def )
{
  short exp = def;
  unsigned long len = sizeof(exp);

  return PrfQueryProfileData( HINI_USERPROFILE,
                              application_name,
                              key, &exp, &len ) ? exp : def;
}

/*--------------------------------------------------
 * Puts unsigned short value to user profile
 *--------------------------------------------------*/
void PMProfile::put_ushort( const char *key, unsigned short exp )
{
  PrfWriteProfileData( HINI_USERPROFILE,
                       application_name,
                       key, &exp, sizeof(exp));
}

/*--------------------------------------------------
 * Retrieves unsigned short value from user profile
 *--------------------------------------------------*/
unsigned short PMProfile::get_ushort( const char *key, unsigned short def )
{
  unsigned short exp = def;
  unsigned long  len = sizeof(exp);

  return PrfQueryProfileData( HINI_USERPROFILE,
                              application_name,
                              key, &exp, &len ) ? exp : def;
}

/*--------------------------------------------------
 * Puts characters string to user profile
 *--------------------------------------------------*/
void PMProfile::put_chars( const char *key, const char* exp )
{
  PrfWriteProfileData( HINI_USERPROFILE,
                       application_name,
                       key, (PVOID)exp, strlen(exp)+1 );
}

/*--------------------------------------------------
 * Retrieves characters string from user profile
 *--------------------------------------------------*/
char* PMProfile::get_chars( const char *key, char* exp, unsigned long len, char* def )
{
  unsigned long get_len = len;

  if( !PrfQueryProfileData( HINI_USERPROFILE,
                            application_name, key, exp, &get_len ))
  {
    strcpy( exp, def );
  }

  return exp;
}
