/* Thread class
 * Copyright (C) 2000 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: pm_thread.cpp,v 1.6 2002/09/11 08:07:55 glass Exp $
 */

#define  INCL_WIN
#define  INCL_DOS
#define  INCL_ERRORS
#include <os2.h>
#include <stdlib.h>

#include "pm_thread.h"
#include "pm_gui.h"
#include "pm_error.h"

/*--------------------------------------------------
 * Constructor
 *-------------------------------------------------*/
PMThread::PMThread()

: run_alive( FALSE ),
  run_id   ( -1    )
{}

/*--------------------------------------------------
 * Thread launch routine
 *-------------------------------------------------*/
void PMThread::launch( PMThread* runable )
{
  PMGUI initialize;

  runable->run_alive = TRUE;
  runable->run();
  runable->run_alive = FALSE;
  runable->run_id    = -1;
}

/*--------------------------------------------------
 * Start the thread
 *-------------------------------------------------*/
PMThread& PMThread::start()
{
  if( !is_alive())
    run_id = _beginthread( (void(*_Optlink)(void*))launch, NULL, 64*1024, this );

  if( run_id == -1 )
    PM_THROW_CLBERROR();

  return *this;
}

/*--------------------------------------------------
 * Stops the thread
 *-------------------------------------------------*/
PMThread& PMThread::stop()
{
  if( is_alive() && tid() != -1 )
  {
    APIRET rc = DosKillThread( tid());

    if( rc != NO_ERROR && rc != ERROR_INVALID_THREADID )
      PM_THROW_OS2ERROR(rc);

    run_alive = FALSE;
    run_id    = -1;
  }
  return *this;
}

/*--------------------------------------------------
 * Wait the thread
 *-------------------------------------------------*/
PMThread& PMThread::join()
{
  if( run_id != -1 )
    DosWaitThread( &run_id, DCWW_WAIT );

  return *this;
}

/*--------------------------------------------------
 * Suspend the thread
 *-------------------------------------------------*/
PMThread& PMThread::suspend()
{
  if( run_id != -1 )
  {
    APIRET rc = DosSuspendThread( run_id );

    if( rc != NO_ERROR )
      PM_THROW_OS2ERROR(rc);
  }
  return *this;
}

/*--------------------------------------------------
 * Resume the thread
 *-------------------------------------------------*/
PMThread& PMThread::resume()
{
  if( run_id != -1 )
  {
    APIRET rc = DosResumeThread( run_id );

    if( rc != NO_ERROR && rc != ERROR_NOT_FROZEN )
      PM_THROW_OS2ERROR(rc);
  }
  return *this;
}

